#!/usr/bin/awk -f

# License: This software is licensed under the MIT or the Apache 2 license.
# Credits: This software includes major portions of JSON.awk, a practical JSON
#   parser written in awk, retrieved on 20180227
#   https://github.com/step-/JSON.awk.git

# Usage:
#  + echo -e "f1\nf2\n" | awk [-v <opt>="val"] [-v <opt>="val"...] -f json-to-uci.awk
#  + { echo; cat f1; } | awk [-v <opt>="val"] [-v <opt>="val"...] -f json-to-uci.awk
#  + awk [-v <opt>="val"] [-v <opt>="val"...] -f json-to-uci.awk f1 [f2...]
# Options: (default value in braces)
#  UCI_PKG="<pkg_name>"	The UCI package name {""}
#  UCI_CFG="<cfg_dir>"	The UCI configuration directory {""}

BEGIN { #{{{
	# Default values for options
	if (UCI_PKG == "") {
		UCI_PKG=""
	}
	if (UCI_CFG == "") {
		UCI_CFG=""
	} else {
		UCI_CFG="-c " UCI_CFG " "
	}

	# for each input file:
	#   TOKENS[], NTOKENS, ITOKENS - tokens after tokenize()
	# at script exit:
	#   FAILS[] - maps names of invalid files to logged error lines
	delete FAILS

	if (1 == ARGC) {
		# file pathnames from stdin
		# usage: echo -e "f1\nf2\n" | awk -f json-to-uci.awk
		# usage: { echo; cat f1; } | awk -f json-to-uci.awk
		while (getline ARGV[++ARGC] < "/dev/stdin") {
			if (ARGV[ARGC] == "")
				break
		}
	} # else usage: awk -f json-to-uci.awk f1 [f2...]

	# set file slurping mode
	srand(); RS="n/o/m/a/t/c/h" rand()
}
#}}}

{ # main loop: process each file in turn {{{
	reset() # See important application note in reset()

	tokenize($0)
	parse()
}
#}}}

END { # process invalid files {{{
	for(name in FAILS) {
		print "invalid: " name
		print FAILS[name]
	}
}
#}}}

function get_token() { #{{{
	TOKEN = TOKENS[++ITOKENS]
	return ITOKENS < NTOKENS
}
#}}}

function parse_array(a1,   idx,ary,ret) { #{{{
	idx=0
	ary=""
	IN_ARRAY=1
	get_token()
#scream("parse_array(" a1 ") TOKEN=" TOKEN)
	if (TOKEN == "]") {
		VALUE=""
		IN_ARRAY=0
		return 0
	}
	while (1) {
		if (ret = parse_value(a1)) {
			IN_ARRAY=0
			return ret
		}
		idx=idx+1
		ary=ary VALUE
		get_token()
		if (TOKEN == "]") {
			break
		} else if (TOKEN == ",") {
			ary = ary ","
		} else {
			report(", or ]", TOKEN ? TOKEN : "EOF")
			IN_ARRAY=0
			return 2
		}
		get_token()
	}
	VALUE=""
	IN_ARRAY=0
	return 0
}
#}}}

function parse_object(a1,   key,obj) { #{{{
	obj=""
	get_token()
#scream("parse_object(" a1 ") TOKEN=" TOKEN)
	if (TOKEN == "}") {
		VALUE=""
		return 0
	}
	while (1) {
		if (TOKEN ~ /^".*"$/) {
			key=TOKEN
		} else {
			report("string", TOKEN ? TOKEN : "EOF")
			return 3
		}
		get_token()
		if (TOKEN != ":") {
			report(":", TOKEN ? TOKEN : "EOF")
			return 4
		}
		get_token()
		if (LEVEL == 1) {
			SAVED_KEY=key
			key=""
		}
		if (LEVEL == 2) {
			gsub(/\"/,"", key)
			printf("%s%s %s.%s=%s\n", UCI_CFG, "set", UCI_PKG, key,
			       SAVED_KEY)
		}
		if (parse_value(a1, key)) {
			return 5
		}
		obj=obj key ":" VALUE
		get_token()
		if (TOKEN == "}") {
			LEVEL--
			break
		} else if (TOKEN == ",") {
			obj=obj ","
		} else {
			report(", or }", TOKEN ? TOKEN : "EOF")
			return 6
		}
		get_token()
	}
	VALUE=""
	return 0
}
#}}}

function parse_value(a1, a2,   jpath,ret,pref,x) { #{{{
	gsub(/\"/,"", a1)
	gsub(/\"/,"", a2)
	jpath=(a1!="" ? a2!="" ? a1 "." : a1 : "") a2 # "${1:+$1,}$2"
#scream("parse_value(" a1 "," a2 ") TOKEN=" TOKEN ", jpath=" jpath)
	if (TOKEN == "{") {
		LEVEL++
		if (parse_object(jpath)) {
			return 7
		}
	} else if (TOKEN == "[") {
		if (ret = parse_array(jpath)) {
			return ret
		}
	} else if (TOKEN == "") { #test case 20150410 #4
		report("value", "EOF")
		return 9
	} else if (TOKEN ~ /^([^0-9])$/) {
		# At this point, the only valid single-character tokens are digits.
		report("value", TOKEN)
		return 9
	} else {
		VALUE=TOKEN
	}
	if (! ("" == jpath || "" == VALUE)) {
		if (IN_ARRAY == 1) {
			pref=UCI_CFG "add_list"
		} else {
			pref=UCI_CFG "set"
		}
		x=sprintf("%s %s.%s=%s", pref, UCI_PKG, jpath, VALUE)
		print x
	}
	return 0
}
#}}}

function parse(   ret) { #{{{
	get_token()
	LEVEL=0
	if (ret = parse_value()) {
		return ret
	}
	if (get_token()) {
		report("EOF", TOKEN)
		return 11
	}
	return 0
}
#}}}

function report(expected, got,   i,from,to,context) { #{{{
	from = ITOKENS - 10; if (from < 1) from = 1
	to = ITOKENS + 10; if (to > NTOKENS) to = NTOKENS
	for (i = from; i < ITOKENS; i++)
		context = context sprintf("%s ", TOKENS[i])
	context = context "<<" got ">> "
	for (i = ITOKENS + 1; i <= to; i++)
		context = context sprintf("%s ", TOKENS[i])
	scream("expected <" expected "> but got <" got "> at input token " ITOKENS "\n" context)
}
#}}}

function reset() { #{{{
	TOKEN=""; delete TOKENS; NTOKENS=ITOKENS=0
	VALUE=""
}
#}}}

function scream(msg) { #{{{
	FAILS[FILENAME] = FAILS[FILENAME] (FAILS[FILENAME]!="" ? "\n" : "") msg
	msg = FILENAME ": " msg
	print msg >"/dev/stderr"
}
#}}}

function tokenize(a1,   pq,pb,ESCAPE,CHAR,STRING,NUMBER,KEYWORD,SPACE) { #{{{
	SPACE="[[:space:]]+"

	gsub(/\"[^[:cntrl:]\"\\]*((\\[^u[:cntrl:]]|\\u[0-9a-fA-F]{4})[^[:cntrl:]\"\\]*)*\"|-?(0|[1-9][0-9]*)([.][0-9]*)?([eE][+-]?[0-9]*)?|null|false|true|[[:space:]]+|./, "\n&", a1)
	gsub("\n" SPACE, "\n", a1)
	sub(/^\n/, "", a1)
	ITOKENS=0 # get_token() helper
	return NTOKENS = split(a1, TOKENS, /\n/)
}
#}}}

# vim:fdm=marker:
