#!/bin/sh
POST_ID="0"
readonly UBUS_JSONRPC="2.0"
readonly UBUS_METHOD="call"
readonly UBUS_NMSP_IDX="2"
readonly UBUS_PROC_IDX="3"
readonly UBUS_SIGN_IDX="4"
_ubus_json_validate() {
	local f="${FUNCNAME[0]}"
	local jsonrpc=""
	local id=""
	local method=""
	json_get_var jsonrpc "jsonrpc"
	if [ -z "${jsonrpc}" ]; then
		jlog_err ${f} "'jsonrpc' version is mandatory!"
		return 1
	fi
	if [ "x${jsonrpc}" != "x${UBUS_JSONRPC}" ]; then
		jlog_err ${f} "'jsonrpc' version '${jsonrpc}' is invalid!"
		return 1
	fi
	json_get_var id "id"
	if [ -z "${id}" ]; then
		jlog_err ${f} "'id' is mandatory!"
		jlog_err ${f} "'id' '${id}' is invalid!"
		return 2
	fi
	utils_is_num "${id}" || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "'id' '${id}' must be numeric!"
		return 2
	fi
	json_get_var method "method"
	if [ -z "${method}" ]; then
		jlog_err ${f} "'method' is mandatory!"
		return 3
	fi
	if [ "x${method}" != "x${UBUS_METHOD}" ]; then
		jlog_err ${f} "'method' '${method}' is invalid!"
		return 3
	fi
	if ! json_is_a "params" array; then
		jlog_err ${f} "'params' is mandatory and must be an array!"
		return 4
	fi
	return 0
}
_ubus_nmsp_get() {
	local retv="${1}"
	local f="${FUNCNAME[0]}"
	local ret=0
	local jnmsp=""
	utils_check_retv "${retv}" "jnmsp" || return $?
	json_get_var jnmsp "${UBUS_NMSP_IDX}"
	if [ -z "${jnmsp}" ]; then
		jlog_err ${f} "Could not get 'namespace' from the request!"
		return 1
	fi
	slog_inf "${f}" "Namespace: '${jnmsp}'"
	eval "${retv}=\${jnmsp}"
	return 0
}
_ubus_proc_get() {
	local retv="${1}"
	local f="${FUNCNAME[0]}"
	local ret=0
	local jproc=""
	utils_check_retv "${retv}" "jproc" || return $?
	json_get_var jproc "${UBUS_PROC_IDX}"
	if [ -z "${jproc}" ]; then
		jlog_err ${f} "Could not get 'procedure' from the request!"
		return 1
	fi
	slog_inf "${f}" "Namespace: '${jproc}'"
	eval "${retv}=\${jproc}"
	return 0
}
ubus_sign_get() {
	local json="${1}"
	local retv="${2}"
	local f="${FUNCNAME[0]}"
	local ret=0
	local idx
	local jsign=""
	utils_check_retv "${retv}" "json" "idx" "jsign" || return $?
	if [ -z "${json}" ]; then
		jlog_err ${f} "Missing mandatory arguments!"
		return 1
	fi
	idx=$((${UBUS_SIGN_IDX} - 1))
	jsign=$(jq '.params['${idx}']' 2>&1 <<< "${json}") || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error '${ret}' getting Signature!"
		return ${ret}
	fi
	if [ ${BE_SLOG} -eq 1 ]; then
		utils_cmd_exists "jq" || return 1
		slog_inf "${f}" "Signature: '$(jq -c . <<< ${jsign})'"
	fi
	eval "${retv}=\${jsign}"
	return 0
}
ubus_build_post() {
	local nmsp="${1}"
	local proc="${2}"
	local sign="${3}"
	local retv="${4}"
	local f="${FUNCNAME[0]}"
	local ret=0
	local json
	utils_check_retv "${retv}" "nmsp" "proc" "sign" "json" || return $?
	if [ -z "${nmsp}" ] || [ -z "${proc}" ] || [ -z "${retv}" ]; then
		jlog_err ${f} "Missing mandatory arguments!"
		err_code=1
	fi
	[ -z "${sign}" ] && sign='{}'
	json='{'
	json="${json}"'"jsonrpc":"'"${UBUS_JSONRPC}"'",'
	json="${json}"'"id":"'"${POST_ID}"'",'
	json="${json}"'"method":"'"${UBUS_METHOD}"'",'
	json="${json}"'"params":["","'"${nmsp}"'","'"${proc}"'",'"${sign}"']'
	json="${json}"'}'
	slog_inf "${f}" "'${json}'"
	eval "${retv}=\${json}"
	return 0
}
ubus_out() {
	local err_code="${1}"
	local data="${2}"
	local f="${FUNCNAME[0]}"
	local resp
	if [ -z "${err_code}" ]; then
		jlog_err ${f} "Missing mandatory arguments!"
		err_code=1
	fi
	http_hdr_ubus
	resp='{'
	resp="${resp}"'"jsonrpc":"'"${UBUS_JSONRPC}"'",'
	resp="${resp}"'"id":"'"${POST_ID}"'",'
	resp="${resp}"'"result":['"${err_code}"','
	if [ "${err_code}" == "0" ]; then
		[ -z "${data}" ] && data='{"status":"OK"}'
	else
		data=$(jlog_dump)
		[ -z "${data}" ] && data='{"status":"Error"}'
	fi
	resp="${resp}""${data}"']'
	resp="${resp}"'}'
	echo ${resp}
	if [ ${BE_SLOG} -eq 1 ]; then
		slog_inf "${f}" "'$(jq -c . <<< ${resp})'"
	fi
	exit 0
}
do_ubus() {
	local json="${1}"
	local f="${FUNCNAME[0]}"
	local ret=0
	local res=""
	local nmsp=""
	local proc=""
	utils_cmd_exists "jq" || ubus_out $?
	if [ -z "${json}" ]; then
		jlog_err ${f} "Missing mandatory arguments!"
		return 1
	fi
	utils_json_load "${json}" || ubus_out $?
	_ubus_json_validate "${json}" || ubus_out $?
	slog_inf "${f}" "JSON received in POST body is syntax proof"
	json_get_var POST_ID "id"
	json_select "params" || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Could not select 'params' object!"
		ubus_out ${ret}
	fi
	_ubus_nmsp_get nmsp || ubus_out $?
	_ubus_proc_get proc || ubus_out $?
	case "${nmsp}" in
	"instreamer4")
		[ -z "${INC_INSTRM}" ] && . ${SH}/be_instrm.sh
		do_instrm "${proc}" res || ret=$?
		;;
	"login")
		[ -z "${INC_LOGIN}" ] && . ${SH}/be_login.sh
		do_login "${proc}" res || ret=$?
		;;
	"network")
		[ -z "${INC_NET}" ] && . ${SH}/be_net.sh
		do_net "${proc}" res || ret=$?
		;;
	"system")
		[ -z "${INC_SYS}" ] && . ${SH}/be_sys.sh
		do_sys "${proc}" res || ret=$?
		;;
	"uci")
		[ -z "${INC_UCI}" ] && . ${SH}/be_uci.sh
		do_uci "${proc}" res || ret=$?
		;;
	"barix_webui")
		res=$(. ${SH}/barix_webui.sh "call" "${proc}") || ret=$?
		;;
	"version")
		[ -z "${INC_VERSION}" ] && . ${SH}/be_version.sh
		do_version "${proc}" res || ret=$?
		;;
	*)
		jlog_err "${f}" "Namespace '${nmsp}' is invalid!"
		ret=1
		;;
	esac
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error occurred in '${nmsp}' namespace!"
	fi
	json_cleanup
	ubus_out "${ret}" "${res}"
	exit 0
}
INC_UBUS=1
